-- thanks to sweetman for contributing parts of the race store code

races = {}
racescount = 0
currentrace = ""
cache_file = getSetting("Cache Path") .. "race_scores_bonneville.dat"
log("race system using scores file: "..cache_file)

function writeScore(racename,score)
	local f = io.open(cache_file, "a+")
	f:write(racename.."|"..score.."\n")
	f:close()
	log('race score file written')
end

function findScore(racename)
	local f = io.open(cache_file, "r")
	minval = 9999
	if not f == nil then
		for line in f:lines()
		do
			line_ar = explode('|', line)
			if line_ar[1] == racename or tonumber(line_ar[2]) < minval then
				minval = tonumber(rcdt[2])
			end
		end
		log('race score file read')
	else
		log('race score file not existant')
	end
	return minval
end

function setupRace(racename, positions, loop)
	for a,v in ipairs(positions) do
		oname = 'bonneville-chp'
		if a == table.getn(positions) and not loop then
			-- put the finish thing at the end, not a normal checkpoint, only on non-loop races
			oname = 'bonneville-chp'		
		end
		if #v == 7 then
			oname = v[7]
		end
		iname = 'race_'..racename..'_'..a
		log("*creating object '"..oname.."' with name '"..iname.."'.")
		spawnObject(oname, iname, v[1],v[2],v[3], v[4],v[5],v[6], raceEventHandler)
	end
	-- add the race finally
	races[racename] = {}
	races[racename]['besttime'] = findScore(racename)
	races[racename]['racename'] = racename
	races[racename]['positions'] = positions
	races[racename]['isloop'] = loop
	races[racename]['startTime'] = 0
	races[racename]['lastcheckpoint'] = 1
	races[racename]['checkpointspassed'] = 1
	races[racename]['numcheckpoints'] = table.getn(positions)
	if loop then
		races[racename]['goalnum'] = 1
	else
		races[racename]['goalnum'] = table.getn(positions)
	end
	
end

function getCheckPointFromName(instance)
	ar = explode('_', instance)
	return tonumber(ar[3])
end

function getRacenameFromName(instance)
	ar = explode('_', instance)
	return tostring(ar[2])
end

function setupArrow(position)
	--log("reposition arrow: "..position)
	txt = currentrace.."\ncheckpoint "..tostring(position).." / "..races[currentrace]['numcheckpoints']
	if position == 1 and races[currentrace]['isloop'] then
		txt = "Finish"
	end
	if position == -1 then
		-- hide it
		pointArrow(0, 0, 0, "")
	else
		data = races[currentrace]['positions'][position]
		if data then
			pointArrow(data[1], data[2], data[3], txt)
		else
			pointArrow(0, 0, 0, "")
		end
	end
end

function raceEventHandler(instance, box)
	

	checknum = getCheckPointFromName(instance)
	racename = getRacenameFromName(instance)
	
	-- create an array with all required names in it
	if currentrace == "" then
		-- not racing currently
		if checknum == 1 then
			-- start of the race
			currentrace = racename
			--log(races[currentrace]['racename'])
			races[currentrace]['startTime'] = getTime()
			races[racename]['lastcheckpoint'] = 1
			races[currentrace]['checkpointspassed'] = 1
			setupArrow(2)
			startTimer()
			flashMessage("Land speed record attempt started!", 10)
		--else
			-- passed some not-start checkpoint
			--flashMessage("This is checkpoint "..checknum.." of the Race "..racename.."!", 4)
		end
	elseif currentrace == racename then

		-- we hit a checkpoint from the same race!
		if checknum == races[currentrace]['lastcheckpoint'] + 1 then
			-- racing, correct checkpoint!
			if checknum == races[currentrace]['numcheckpoints'] and races[currentrace]['isloop'] then
				setupArrow(1)
			else
				setupArrow(checknum+1)
			end
			races[currentrace]['lastcheckpoint'] = checknum
			races[currentrace]['checkpointspassed'] = races[currentrace]['checkpointspassed'] + 1
			time = stopTimer()
			speed = 1 / (time / 3600)
			startTimer()
			msg = "Speed at the kilometer was "..string.format("%.1f", speed).." km/h"
			flashMessage(msg, 20)
		else
			if not checknum == races[currentrace]['lastcheckpoint'] then
				-- racing, wrong checkpoint, prevent trigger of the same checkpoint twice
				flashMessage("Wrong gate! You must find and pass Checkpoint "..races[currentrace]['lastcheckpoint'] + 1, 4)
			end
		end
		if checknum == races[currentrace]['goalnum'] and races[currentrace]['checkpointspassed'] == races[racename]['numcheckpoints'] then
			-- racing, reached finish
			time = stopTimer()
			-- speed = 1 / (time / 3600)
			-- msg = "Your speed was "..string.format("%.1f", speed).." km/h"
		
			
			setupArrow(-1)
			-- reset some values
			races[currentrace]['checkpointspassed'] = 0
			
			currentrace = ""
			-- flash finish message for 10 seconds
			flashMessage(msg, 30)
		end
	else
		-- flashMessage("This Checkpoint belongs to race"..racename.."!", 4)
	end

end


-- 36807, 39.02, 35748, 0.0, 45.0, 0.0,    0.8,  0,            0,             flat
-- 49396, 39.02, 24787, 0.0, 45.0, 0.0,    0.8,  0,            0,             flat
-- 12589, 00,   -10961
-- x = -0,870682 y
-- tan alfa = 1,1485265942888422589179819359547
-- alfa = 48,954538
-- x = sin alfa * 1000 m = 754,188785
-- y = cos alfa * 1000 m = -656,65766

race_landspeed = {}
for trap = 1, 17, 1 do
	
	xvalue = 36807.000000 + (754.188785 * (trap - 1))
	yvalue = 35748.000000 - (656.657660 * (trap - 1))
	race_landspeed[trap] = {xvalue, 37.0, yvalue, 0.0, 131.0, 0.0}

	xvalue = 36787.000000 + (754.188785 * (trap - 1))
	yvalue = 35728.000000 - (656.657660 * (trap - 1))
	spawnObject("b_flag", "bflag", xvalue, 39.0, yvalue, 0.0, 41.0, 0.0)
	
	xvalue = 36827.000000 + (754.188785 * (trap - 1))
	yvalue = 35768.000000 - (656.657660 * (trap - 1))
	spawnObject("b_flag", "bflag", xvalue, 39.0, yvalue, 0.0, 41.0, 0.0)
end

for cones = 1, 5, 1 do
	-- x = sin 49 * 20 = 15.80
	-- y = 13.12
	xvalue = 36807.000000 - (15.80 * (cones - 1))
	yvalue = 35748.000000 + (13.12 * (cones - 1))
	spawnObject("b_cone", "bcone", xvalue, 39.0, yvalue, 0.0, 41.0, 0.0)
end

for cones = 1, 5, 1 do
	xvalue = 49396.000000 + (15.80 * (cones - 1))
	yvalue = 24787.000000 - (13.12 * (cones - 1))
	spawnObject("b_cone", "bcone", xvalue, 39.0, yvalue, 0.0, 41.0, 0.0)
end


function init()
	setupRace("Landspeed", race_landspeed, false)
	log("bonneville.terrn.lua file loaded")	
end

init()